/* "header.h": the grand header file of the project "multicomp" */
/* Time-stamp: <97/12/19 11:29:17 hgs> */
          
/* Notes: 
1. This project contains files:
main.c       : main function 
struct.c     : data structures
simulate.c   : code for simulation
menu.c       : input menu system   ( a monster )
compute.c    : numerical computing facilities
boundary.c   : boundary computation
utility.c    : input/output facilities
 
2. For consistency, every array and matrix starts from 0.    */

/* Global constants */
#define TRIALSIZE 10000 /* number of trials generated in one simulation */
#define UPPER  1        /* indicate type of boundary hit */
#define LOWER -1
#define MAXN  500       /* the largest possible Nmax */
#define MAX_ELEMENT 50  /* the ceiling of the number of elementary events */
#define MAXP  25        /* the maximum number of true prob vectors. */
#define EFFICACY  1     /* something for readability */
#define ADVERSE   0
#define SUCCEED  1 
#define FAIL     0 
#define YES  1
#define NO   0
#define TEST  NO 
#define Max(A, B) ((A) > (B)?  (A) : (B))
#define Min(A, B) ((A) < (B)?  (A) : (B))

/* data structure */ 
typedef struct design {
    int Nmin;  /* minimum number of patients in the trial */ 
    int Nmax;  /* maximum number of patients in the trial */
    int number_of_element;  /* number of elementary events */
    int number_of_event;    /* number of compound events */
    int number_of_cond_event; /* number of conditional event */
    int sparameter_type;   /* type of parameterization of thetaS */
    int Sprior_type;       /* prior type of thetaS = number of prior Dir() 
			      single = 1; mixture > 1 */
/* single prior */    
    double *aS; /* Theta_S ~ Dir(aS) */
    double *aE; /* Theta_E ~ Dir(aE) */ /* prior type specified by users */
    double *muS;  double *muE;

/* mixture prior */
    double Sprior_weight[10]; /* the weight vector for the mixture */
    double mix_aS[10][50];    /* mix_aS[i][j] = Dir_i(..aS_j..) */

/* parameters of monitoring */
    int monitor_method;    /* 1 = by cohort;  2 = by time */
    int cohort_size;       /* check with the stopping criteria in batch mode */
    double monitor_period;   /* time to check stopping boundary */
    double accrual_rate;   /* Poisson accrual rate */
    double response_window; 

    int Nprob;           /* number of simulation to run */
    double trueprob[MAXP][MAX_ELEMENT]; /* store the true prob vector for 
					   simulation. */
   /* double **trueprob; */		
  /*  struct event **event; */   /* list of compound and conditional events */
    struct event *event0;        /* list of compound and conditional events */
    struct event *current_event; /* pointer to the event the program working on */

    char filename[200];      /* output file name */
} 
Design;
/*----------------------------------------------------------*/
/* one set of stopping rules defined on one compound events */
typedef struct event {
    /* given */
    int type;     /* 0= Adverse; 1= Efficacy; 2=unknown*/ /* maybe unnecessary */
    char name[50];
    int *definition; /* an array of length K(# of elementary outcome) */
    int Nmax;     int Nmin;  int number_of_element;
    double aS;   double bS;     /* etaS ~ beta(aS, bS) */
    double aE;   double bE;     /* etaE ~ beta(aE, bE) */
    
    double mix_aS[10]; 
    double mix_bS[10]; /* for mixture prior only */

    double deltaL;    double deltaU;
    double pL;        double pU;

    /* computed */
    int out;  /* outcome counts. should reset to 0 for a new simulation */
    int outcome[10];  /* event count for k_th arm = outcome[k] */
    int *lowerbound;  /* an array of length Nmax */
    int *upperbound;  /* an array of length Nmax */
    int boundary_type; /* 1=lower 2=both 3=upper 4=none*/

    /* linking conditional events is tricky! very interesting.         */ 
    int child_count;  /* number of conditional events defined on it    */
    struct event *child;      /* conditional event: (child | event)    */
    struct event *next;       /* sibling relationship. non-conditional */
    /*    struct event *last;*/  /* maybe unnecessary, decide it later */

    struct design *design;    /* where this event belongs to */


    /* for conditional event only */
    struct event *parent;     /* conditional event (event|parent) is 
				 itself an event. */
    struct event *seed;   /* this is for remove_event()
			     cond event : (seed | parent) */
}
Event;
 
/* global object */
Design Template_Design; 

/* global variables */
int NMAX, NMIN, NUMBER_OF_ELEMENT; 

int BOUND[MAXN][MAXN];  /* boundary for two-arm comparison */
/*--------------------------------------------------------------*
 * function list                                                *
 *--------------------------------------------------------------*/
/* "struct.c"  ------------------------------------------------ */
void initial_design(Design*, int Nmin, int Nmax, 
		    int num_element, char *filename);

int retrieve_old_design(char *name, Design*);
Event *retrieve_event(char*); 
void retrieve_cond_event(char*, Design*);

/* could be obsolete */
Design *make_design(int Nmin, int Nmax, double *aS, double *aE, 
		    int num_element, int cohort_size, double monitor, 
		    double accrual_rate, char *filename);

void add_event(Event*, Design*); /* fill in the list of events */
void remove_event(Event*, Design*); /* remove an event from the design */

/* list all events and set design->current_event 
   to the event choosen by the user */
void traverse_event_list(Design*);
void free_design(Design*);
Event *make_event(int type, char *name, int Nmax, int element, 
		  int *definition, double *aS, double *aE, 
		  double deltaL, double deltaU, 
		  double pL, double pU);
void free_event(Event *ePtr);

/* conditional event is itself an event */
/* make one, and link to its "parent" */
void make_conditional_event(Event *child, Event *parent, int *flag); 
			
void remove_conditional_event(Event*); 
int check_conditional(Event *child, Event *parent);


/* "simulate.c" ---------------------------------------------------------*/
 
void simulation(Design*);
void print_primary_result_in_file(Design*, double*, int **result);
void print_secondary_result_in_file(Design*, double*, int **result);

/* simulation for trueprob */
void simulate(Design*, double *trueprob, int **result, int*);
void run_trial(int, Design*, double* trueprob);
void update_event_count(int k, int n, Event*); /* update outcome[] */
int check_bound(int, int, Design*, int* result);
int intcompare(const void*, const void*);  /* for the use of qsort() */
void print_overhead(Design*);
int power(int base, int j);

/* "boundary.c" ---------------------------------------------------------*/

void compute_design_boundary(Design*);
void compute_event_upperbound(Event*);
void compute_event_lowerbound(Event*);
void read_design_boundary(Design*);
void read_event_boundary(Event*);

void input_boundary_parameter(Event*);
void review_boundary_parameter(Design*);

/* work horses: compute Pr[thetaS + delta < thetaE | x,n]  */
double Lambda_mix(int x, int n, double *aS,double *bS, double *weight,
		  double aE, double bE, double delta);
/* for phase2 A specifically */
double Lambda3(int x, int n, double aE, double bE, double p_Null);
double Lambda4(int x,int n, double aS,double bS,
	       double aE, double bE, double delta);
/* for extended randomized trial */
double Lambda5(int x1, int x2, int n1, int n2, double ae1, double be1, 
	       double ae2, double be2, double delta);

/* generate x, f(x) data ready for Splus plot() */
void plot_func(double a, double b, int x, int n, 
	       Event* event,  double (*func)(), char *filename);

/* "compute.c" ------------------------------------------------------------*/
/* integrator */
/*double integrator2(double (*fun)(), int X, int n, Event*, double delta,
		  double lower, double upper); */
int quanc8(double (*fun)(), int X, int n, double aS, double bS, 
	   double aE, double bE, double delta,
	   double *a, double *b, double *aerr, double *rerr,
	   double *result, double *errest, int *nofun, double *flag);

void new_quanc8(double (*fun)(), double a, double b, 
	       double abserr, double relerr, double *result,
	       double *errest, int* nofun, double *flag);

/* gaussian quarature: used in the original version of multicomp */
/* double gaussquad(double (*fun)(), int x, int n, double aS, double bS, 
   double aE, double bE, double delta,
   double lower, double upper);
*/
/* for beta distribution */
double pbeta(double, double, double);    
double dbeta(double, double, double); 
double qbeta(double, double, double); 

/* reparameterization */
/* from mean, width, and cover prob to Dirichlet a[i]  */
/* flag = 1: standard prior;  flag = 2: experimental prior */
void getdir(Design*, double width, double coverprob, int flag);

/* compute Nmax */
int compute_Nmax(Event*, double mu, double width, double coverprob);

/* ranking */
void qrank(double*, int*, int, int);
void dswap(double*, int, int);
void iswap(int*, int, int);

/* balanced randomization */
int *balrand(int k, int n);
int re_assign(int k, int total_arms, int n, int N, int *assign, int *hit);
/* void randomized_trial(int K, Design *design, Event *target, 
   double **trueprob, int **result); */
void randomized_trial( Design*, int *reassign, int *extended );

/* "utility.c" ------------------------------------------------------------*/
/* Output facilities */
void print_design_on_screen(Design*);
void print_design_in_file(Design*, int*);
void print_event_on_screen(Event*);
void print_event_in_file(Event*, FILE*);
void print_boundary_in_file(Design*);
void print_boudnary_on_screen(Design*);
void print_event_boundary_in_file(Event*, FILE*);
void print_event_boundary_on_screen(Event*);
void print_event_list(Design *design);

/* Input facilities */
int get_an_integer(void);
/* char *get_a_word(int size_of_word); */
double get_a_double(double, double);
char *get_file(char* filename);
int get_data(char*, char*, char*); 
/* char *get_data(char*, char*); */

/* move currentPtr to the event chosen */
int get_event(int index, Design*); 
void choose_event(Design*); /* get_event() + interface */


/* matrix manipulation */
int **imatrix(int nrow, int ncol);
double **dmatrix(int nrow, int ncol);
void free_matrix(void **matrix);
void print_imatrix(int **matrix, int nrow, int ncol);
void print_dmatrix(double **matrix, int nrow, int ncol);

/* "menu.c" ---------------------------------------------------------------*/
/* Input menu system */
void menu(int); 

/* for modifying old design */
void update_Nmax(Design*, int);
void update_aSaE(Event *event);
void rename_design(Design*, int*);
void copy_design(Design*, int*);
void update_event(Event*);

void plot_menu(Design*);
/* stupid code for phase IIA */ /* must fix it later */
void phaseIIA(int Nmin, int Nmax, double pNull, double pL, 
	      double aE, double bE, FILE*);
void print_stopping_bound(Event *event, FILE*);
void phase2a_table(int);
void run_phase2a(void);



/* the latest, better code */
int *phase2a_lowerbound(double pNull, double pL, double a, double b, int Nmax);
int *phase2a_upperbound(double pNull, double pU, double a, double b, int Nmax);
/* nmin = 1 */
void phase2a(double, double, double, double, double, int, FILE*, int);
/* allows nmin */
void phase2a2(double, double, double, double, double, int, int, FILE*, int);

/* "iofile.c"----------------------------------------------------------------*/
/* char *put_data(char* text, char* token); */







